/*  
DummyWrapper.cpp - A C++ class for wrapping PGSL C routines

Copyright (C) Benny Raphael  

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or  any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


*/
#include "StdAfx.h"
#include "DummyWrapper.h"


#include "memory.c"
#include "error.c"
#include "PGSL.c"

using namespace PGSLMain;
using namespace PGSLVBCntrl;


double objectiveFunction(struct ProblemSetup *setup, double *paramValues){

  int i;
  int needcopy=0;
  double value=0;
  static double debug = 0;
  PGSLVBWrapper *optimisationSetup = DummyWrapper::theInstance->optimisationSetup;

  /* Getting the values of parameters from the array.  */
  for (i=0; i < setup->numVars; i++) {
	  optimisationSetup->set_currentPoint(i,paramValues[i]);
  }

  /*  Calculating the value of the objective function  */
  value = optimisationSetup->objectiveFunction();

  optimisationSetup->numEvaluations = setup->numEvaluations;
  if (setup->numEvaluations == 0) needcopy = 1;
  else {
	  if (optimisationSetup->globalMinimum > value) needcopy = 1;
  }

  if (needcopy == 1) {
	  if (debug == 1)  {
		  printf("%lf\n", value);
	  }
	  for (i=0; i < setup->numVars; i++) optimisationSetup->set_minimumPoint(i, paramValues[i]);
	  optimisationSetup->globalMinimum = value;
  }
  return value;
}

double doMinimise(PGSLVBWrapper *optimisationSetup) {

   /*  Structure representing the optimisation problem in C */
   struct ProblemSetup *setup;
   int i;
   double minvalue;
   double xmin, xmax, precision;
   PGSLVBWrapper::Axis *axis;
	
	/*  Setting up the problem */
   setup = ProblemSetup_create(optimisationSetup->numVariables, optimisationSetup->NFC, optimisationSetup->NSDC, optimisationSetup->threshold);
   setup->costFunction = objectiveFunction;
   for (i=0; i<optimisationSetup->numVariables; i++) {
	   axis = optimisationSetup->get_axes(i);
	   xmin = axis->min;
	   xmax = axis->max;
	   precision = axis->precision;
       setup->axes[i] = PAxis_create(xmin,  xmax);
	   setup->axes[i]->axisPrecision = precision;
   }
   setup->suppressOutput = 2;

   /* Calling PGSL  */
   minvalue = findMinimum(setup);

   return minvalue;


}

void DummyWrapper::runOptimisation(PGSLVBWrapper *theOptimisationSetup) {
	optimisationSetup = theOptimisationSetup;
	theInstance = this;
	doMinimise(optimisationSetup);
	theOptimisationSetup->optimisationCompleted();

}

