'
' This class belongs to PGSL VB Version
' 
' Copyright (C) Benny Raphael  
' 
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or  any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
' 
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


' This is a sample class to denote how a PGSL problem is defined in the VB version
' This implements the Griewank's function F8
Public Class F8
    Inherits PGSLVBWrapper

    Sub New()
        Dim xmin As Double = -512
        Dim xmax As Double = 511
        Dim xprecision As Double = 0.0001
        Dim i As Integer

        ' Perform any other initialisation of variables here

        MyBase.numVariables = 50
        MyBase.threshold = 0.0001


        For i = 0 To numVariables - 1
            MyBase.axes(i) = New Axis(xmin, xmax, xprecision)   ' Specifying the bounds and precision for the each variable
        Next
        MyBase.NS = 2
        MyBase.NPUC = 1
        MyBase.NFC = 60 * numVariables  ' F8 requires lot of evaluations to converge
        MyBase.NSDC = 100

    End Sub

    ' This is the objective function to be minimised
    ' the argument point contains the values of variables suggested by PGSL.  The user should 
    ' compute and return the evaluation of the point as a double
    Public Overrides Function objectiveFunction() As Double
        Dim x1 As Double
        Dim costerm As Double
        Dim total As Double
        Dim i As Integer

        total = 1
        For i = 0 To numVariables - 1
            x1 = currentPoint(i)
            total += x1 * x1 / 4000.0
        Next i

        costerm = 1
        For i = 0 To numVariables - 1
            x1 = currentPoint(i)
            costerm *= Math.Cos(x1 / Math.Sqrt(i + 1))
        Next i

        total -= costerm
        objectiveFunction = total

    End Function

    ' this function is called by PGSL after the completion of the optimisation
    ' Use this function to perform any saving of results 
    Public Overrides Function optimisationCompleted() As Integer

        'This is how the results are obtained
        Dim bestcost As Double = MyBase.globalMinimum
        Dim x1 As Double = MyBase.minimumPoint(0)
        Dim x2 As Double = MyBase.minimumPoint(1)
        Dim x3 As Double = MyBase.minimumPoint(2)
        Dim totalNumEvaluations = MyBase.numEvaluations

        ' Save results here

    End Function
End Class
