'
' This class belongs to PGSL VB Version
' 
' Copyright (C) Benny Raphael  
' 
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or  any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
' 
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



'  This is a wrapper class to define the PGSL problem in VB
Public MustInherit Class PGSLVBWrapper
    Private intNumVariables As Integer ' The number of variables in the problem
    Private arAxes() As Axis
    Private intNS As Integer = 2
    Private intNFC As Integer
    Private intNSDC As Integer
    Private intNPUC As Integer = 1
    Private dblThreshold As Double
    Private dblCurrentPoint() As Double
    Private dblGlobalMinimum As Double
    Private dblMinimumPoint() As Double
    Private intNumEvaluations As Integer = 0

    '  The class represents a variable axis
    Public Class Axis
        Private dblMin As Double
        Private dblMax As Double
        Private dblPrecision As Double

        ' lower bound of the axis
        Public Property min() As Double
            Get
                Return dblMin
            End Get
            Set(ByVal Value As Double)
                dblMin = Value
            End Set
        End Property

        ' upper bound of the axis
        Public Property max() As Double
            Get
                Return dblMax
            End Get
            Set(ByVal Value As Double)
                dblMax = Value
            End Set
        End Property

        ' precision of the axis
        Public Property precision() As Double
            Get
                Return dblPrecision
            End Get
            Set(ByVal Value As Double)
                dblPrecision = Value
            End Set
        End Property

        Public Sub New(ByVal minbound As Double, ByVal maxbound As Double, ByVal axisprecision As Double)
            dblMin = minbound
            dblMax = maxbound
            dblPrecision = axisprecision
        End Sub

    End Class



    ' This property is used to set the axes for each variable
    Public Property axes(ByVal index As Integer) As Axis
        Get
            If (arAxes Is Nothing) Then
                Throw New Exception("The property numVariables should be set before accessing axes")
            End If
            Return arAxes(index)
        End Get

        Set(ByVal Value As Axis)
            If (arAxes Is Nothing) Then
                Throw New Exception("The property numVariables should be set before accessing axes")
            End If
            arAxes(index) = Value
        End Set
    End Property


    ' Number of iterations in the sampling cycle
    Public Property NS() As Integer
        Get
            Return intNS
        End Get
        Set(ByVal Value As Integer)
            intNS = Value
        End Set
    End Property

    ' Number of iterations in the probability updating cycle
    Public Property NPUC() As Integer
        Get
            Return intNPUC
        End Get
        Set(ByVal Value As Integer)
            intNPUC = Value
        End Set
    End Property

    ' Number of iterations in the focusing cycle
    Public Property NFC() As Integer
        Get
            Return intNFC
        End Get
        Set(ByVal Value As Integer)
            intNFC = Value
        End Set
    End Property

    ' Number of iterations in the subdomain cycle
    Public Property NSDC() As Integer
        Get
            Return intNSDC
        End Get
        Set(ByVal Value As Integer)
            intNSDC = Value
        End Set
    End Property

    ' threshold of the objective function.  Use an impossible low value if the lower bound of the objective
    ' function is not known
    Public Property threshold() As Double
        Get
            Return dblThreshold
        End Get
        Set(ByVal Value As Double)
            dblThreshold = Value
        End Set
    End Property

    ' Number of variables in the problem
    ' this should be set before setting any other property
    Public Property numVariables() As Integer
        Get
            Return intNumVariables
        End Get
        Set(ByVal Value As Integer)
            intNumVariables = Value
            ReDim arAxes(intNumVariables - 1)
            ReDim dblMinimumPoint(intNumVariables - 1)
            ReDim dblCurrentPoint(intNumVariables - 1)
        End Set
    End Property


    Public Property axisMin(ByVal index As Integer) As Double
        Get
            If (arAxes Is Nothing) Then
                Throw New Exception("The property numVariables should be set before accessing axes")
            End If
            Return arAxes(index).min
        End Get
        Set(ByVal Value As Double)
            arAxes(index).min = Value
        End Set
    End Property


    Public Property axisMax(ByVal index As Integer) As Double
        Get
            If (arAxes Is Nothing) Then
                Throw New Exception("The property numVariables should be set before accessing axes")
            End If
            Return arAxes(index).max
        End Get
        Set(ByVal Value As Double)
            arAxes(index).max = Value
        End Set
    End Property


    Public Property axisPrecision(ByVal index As Integer) As Double
        Get
            If (arAxes Is Nothing) Then
                Throw New Exception("The property numVariables should be set before accessing axes")
            End If
            Return arAxes(index).precision
        End Get
        Set(ByVal Value As Double)
            arAxes(index).precision = Value
        End Set
    End Property

    Public Property currentPoint(ByVal index As Integer) As Double
        Get
            Return dblCurrentPoint(index)
        End Get
        Set(ByVal Value As Double)
            dblCurrentPoint(index) = Value
        End Set
    End Property

    Public Property minimumPoint(ByVal index As Integer) As Double
        Get
            Return dblMinimumPoint(index)
        End Get
        Set(ByVal Value As Double)
            dblMinimumPoint(index) = Value
        End Set
    End Property

    Public Property globalMinimum() As Double
        Get
            Return dblGlobalMinimum
        End Get
        Set(ByVal Value As Double)
            dblGlobalMinimum = Value
        End Set
    End Property

    ' Total Number of evaluations of the objective function
    Public Property numEvaluations() As Integer
        Get
            Return intNumEvaluations
        End Get
        Set(ByVal Value As Integer)
            intNumEvaluations = Value
        End Set
    End Property

    ' Subclasses should contain the definition of this function which will be minimised by PGSL
    ' the argument point contains the values of variables suggested by PGSL.  The user should 
    ' compute and return the evaluation of the point as a double
    Public MustOverride Function objectiveFunction() As Double

    ' this function is called by PGSL after the completion of the optimisation
    ' Override this function to perform any saving of results 
    Public Overridable Function optimisationCompleted() As Integer

    End Function


End Class
